@@ -31,7 +31,7 @@ struct Track {
     struct {
       char *filename;
     } data;
-  };
+  } contents;
 };
 
 #define CD_TYPE_RECORDER            (cd_recorder_get_type ())
