@@ -311,7 +311,9 @@ media_type_matches (CDMediaType type, gboolean blank_cd)
 	switch (type)
 	{
 	case CD_MEDIA_TYPE_ERROR:
+		return FALSE;
 	case CD_MEDIA_TYPE_UNKNOWN:
+		return TRUE;
 	case CD_MEDIA_TYPE_CD:
 	case CD_MEDIA_TYPE_DVD:
 	case CD_MEDIA_TYPE_DVD_RAM:
@@ -341,7 +343,8 @@ wait_for_insertion (CDDrive *rec, gboolean blank_cd)
 
 	reload = FALSE;
 	type = guess_media_type (rec->device);
-	if (type == CD_MEDIA_TYPE_UNKNOWN || type == CD_MEDIA_TYPE_ERROR) {
+
+	if (type == CD_MEDIA_TYPE_ERROR) {
 		reload = TRUE;
 	}
 
@@ -534,6 +537,15 @@ estimate_size (char *uri)
 				   (GnomeVFSDirectoryVisitFunc) estimate_size_callback,
 				   &size);
 
+	if (size == 0) {
+		GList *list;
+
+		if (gnome_vfs_directory_list_load (&list, uri, GNOME_VFS_FILE_INFO_FIELDS_NONE) == GNOME_VFS_OK) {
+			size = 1;
+			gnome_vfs_file_info_list_free (list);
+		}
+	}
+
 	return size;
 }
 
@@ -550,16 +562,161 @@ setup_close_button (void)
 	g_signal_connect (close_button, "clicked", (GCallback)gtk_main_quit, NULL);
 }
 
+static void
+track_free (Track *track, gpointer user_data)
+{
+	switch (track->type) {
+	case TRACK_TYPE_DATA:
+		g_free (track->contents.data.filename);
+		break;
+	case TRACK_TYPE_AUDIO:
+		g_free (track->contents.audio.filename);
+		g_free (track->contents.audio.cdtext);
+		break;
+	default:
+		g_warning ("Invalid track type %d", track->type);
+	}
+}
+
+static gboolean 
+overwrite_existing_file (GtkWindow *filesel, const char *filename)
+{
+	GtkWidget *dialog;
+	gint ret;
+
+	dialog = gtk_message_dialog_new (filesel,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 _("A file named \"%s\" already exists.\nDo you want to overwrite it?"),
+					 filename);
+	/* Add Cancel button */
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+	/* Add Overwrite button */
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Overwrite"), GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	ret = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	return (ret == GTK_RESPONSE_YES);
+}
+
+static void
+display_error_dialog (GtkWindow *win, const gchar *msg)
+{
+	GtkWidget *error_dlg;
+	
+	g_return_if_fail (msg != NULL);
+
+	error_dlg = gtk_message_dialog_new (win ? win : NULL,
+					    0,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    msg);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (error_dlg), GTK_RESPONSE_OK);
+	gtk_window_set_resizable (GTK_WINDOW (error_dlg), FALSE);
+	gtk_dialog_run (GTK_DIALOG (error_dlg));
+	gtk_widget_destroy (error_dlg);
+}
+
+static void
+file_chooser_response_cb (GtkWidget *chooser, gint response, gpointer data)
+{
+	gchar *file_name;
+	char **file_name_out;
+
+	file_name_out = data;
+
+	if (response != GTK_RESPONSE_OK) {
+		if (response != GTK_RESPONSE_DELETE_EVENT) {
+			gtk_widget_destroy (chooser);
+		}
+		gtk_main_quit ();
+
+		return;
+	}
+
+	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+
+	if (!file_name)
+		return;
+
+	if (strlen (file_name) < 1) {
+		g_free (file_name);
+		return;
+	}
+
+	/* Change into directory if that's what user selected */
+	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+						     file_name);
+		g_free (file_name);
+	} else {
+		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
+			if (!overwrite_existing_file (GTK_WINDOW (chooser), file_name)) {
+				g_free (file_name);
+				return;
+			}
+		}
+
+		gtk_widget_destroy (GTK_WIDGET (chooser));
+		
+		g_free (*file_name_out);
+		*file_name_out = file_name;
+		
+		gtk_main_quit ();
+	}
+}
+
+static char *
+select_iso_filename (void)
+{
+	GtkWidget *chooser;
+	char *filename;
+
+	chooser = gtk_file_chooser_dialog_new (_("Choose a filename for the cdrom image"),
+					       NULL,
+					       GTK_FILE_CHOOSER_ACTION_SAVE,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser),
+					   "image.iso");
+
+	filename = NULL;
+	g_signal_connect (G_OBJECT (chooser),
+			  "response",
+			  G_CALLBACK(file_chooser_response_cb),
+			  &filename);
+
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+
+	gtk_widget_show (chooser);
+
+	/* Run in a loop untill we get the response from file selector */
+	gtk_main ();
+
+	return filename;
+}
+
 static int
 burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, gboolean music_cd, int speed, gboolean dummy)
 {
 	CDRecorder *cdrecorder;
-	GtkWidget *filesel;
+	GList *tracks;
 	char *filename, *str;
-	const char *f;
 	int fd;
 	int res = RESULT_ERROR;
 
+	tracks = NULL;
 	filename = NULL;
 
 	if (rec->type == CDDRIVE_TYPE_FILE) {
@@ -568,17 +725,36 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			g_print ("Writing ISO onto ISO, you freak!\n");
 			return RESULT_FINISHED;
 		}
-		filesel = gtk_file_selection_new (_("Choose a filename for the CD-ROM image"));
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-						 "~/image.iso");
 
-		res = gtk_dialog_run (GTK_DIALOG (filesel));
-		f = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
-		filename = g_strdup (f);
-		gtk_widget_destroy (filesel);
-		
-		if (res == GTK_RESPONSE_CANCEL) {
+		/* Run the file selector and get the iso file name selected */
+		filename = select_iso_filename ();
+
+		/* No file selected, just return; Else proceed */
+		if (filename == NULL) {
 			return RESULT_CANCEL;
+		} else {
+
+			/* Check if you have permission to create or
+			 * overwrite the file.
+			 */
+			if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+				if (access (filename, W_OK) == -1) {
+					display_error_dialog (NULL, _("Don't have permissions to overwrite file."));
+					g_free (filename);
+					return RESULT_ERROR;
+				}
+			} else {
+				gchar *dir_name;
+				dir_name = g_path_get_dirname (filename);
+
+				if (access (dir_name, W_OK) == -1) {
+					display_error_dialog (NULL, _("Don't have permissions to create file."));
+					g_free (filename);
+					g_free (dir_name);
+					return RESULT_ERROR;
+				}
+				g_free (dir_name);
+			}
 		}
 	} else {
 		if (source_iso == NULL && !music_cd) {
@@ -600,7 +776,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 
 	if (source_iso == NULL && !music_cd) {
 		if (filename != NULL) {
-			res = make_iso (filename, label, FALSE);
+			res = make_iso (filename, label, FALSE, TRUE);
 		} else {
 			res = RESULT_RETRY;
 		}
@@ -611,14 +787,14 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			fd = mkstemp(filename);
 			close (fd);
 
-			res = make_iso (filename, label, FALSE);
+			res = make_iso (filename, label, FALSE, TRUE);
 			if (res == RESULT_RETRY) {
 				g_free (filename);
 				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
 				fd = mkstemp(filename);
 				close (fd);
 
-				res = make_iso (filename, label, TRUE);
+				res = make_iso (filename, label, TRUE, TRUE);
 			}
 
 			if (!res) {
@@ -665,7 +841,18 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-rwcd-request",
 				  G_CALLBACK (insert_rwcd_request_cb),
 				  NULL);
-		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, blank_cd, music_cd, dummy);
+		
+		if (music_cd) {
+			tracks = build_list_of_audio_tracks ();
+		} else {
+			Track *track;
+			track = g_new0 (Track, 1);
+			track->type = TRACK_TYPE_DATA;
+			track->contents.data.filename = g_strdup (filename);
+			tracks = g_list_prepend (tracks, track);
+		}
+		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy, FALSE);
+
 		if (source_iso == NULL) {
 			unlink (filename);
 		}
@@ -683,6 +870,8 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 	
  out:
 	g_free (filename);
+	g_list_foreach (tracks, (GFunc)track_free, NULL);
+	g_list_free (tracks);
 	return res;
 }
 
@@ -802,11 +991,10 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *entry;
 	GtkWidget *reuse;
 	GtkWidget *music_check;
-	struct tm *tm;
 	int size_mb;
 	char *str;
 	char buf[129];
-	time_t now;
+	GDate *now;
 
 	label = glade_xml_get_widget (xml, "top_label");
 
@@ -842,13 +1030,17 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 		gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
 		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
 	} else {
-		time (&now);
-		tm = localtime (&now);
+		now = g_date_new ();
+
+		g_date_set_time (now, time (NULL));
+
 		/*
 		translators: see strftime man page for meaning of %b, %d and %Y
 		the maximum length for this field is 32 characters
 		*/
-		strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), tm);	
+		g_date_strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), now);
+
+		g_date_free (now);
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
 
@@ -956,7 +1148,7 @@ verify_iso (const char *filename, char **iso_label)
 	if (res != BUFFER_SIZE) {
 		return FALSE;
 	}
-	buf[BUFFER_SIZE+1] = '\0';
+	buf[BUFFER_SIZE] = '\0';
 	str = g_strdup (g_strstrip (buf));
 	if (!g_utf8_validate (str, -1, NULL)) {
 		/* Hmm, not UTF-8. Try the current locale. */
