@@ -29,8 +29,6 @@
 #include "cd-drive.h"
 #include "cd-recorder.h"
 
-extern int make_iso (const char *filename,
-		     const char *label);
 extern gint build_list_of_music_arguments (char *uri, int *argc, const char **argv, int max_num_args);
 gboolean is_audio (char *mime_type);
 
